import regex
import ctypes
from PIL import ImageFont

#https://stackoverflow.com/a/47461052/1741346
def escape_string_for_xpath(s):
    if '"' in s and "'" in s:
        return 'concat(%s)' % ", '\"',".join('"%s"' % x for x in s.split('"'))
    if '"' in s:
        return "'%s'" % s
    return '"%s"' % s
	

def find_whole_word(needle, haystack, ignore_case):
    if (ignore_case == True):
        return bool(regex.search(regex.compile(r'\b({0})\b'.format(needle), flags=regex.IGNORECASE), haystack))
    else:
        return bool(regex.search(regex.compile(r'\b({0})\b'.format(needle)), haystack))


def remove_numbers_from_string(string):
    return ''.join([i for i in string if not i.isdigit()])
	
	
#Example: get_pil_text_size('Hello world', 12, 'times.ttf')
def get_pil_text_size(text, font_size, font_name):
    font = ImageFont.truetype(font_name, font_size)
    size = font.getsize(text)
    return size


#Example: get_windows_text_size('Hello world', 12, 'Times New Roman')
def get_windows_text_size(text, font_size, font_name):
    class SIZE(ctypes.Structure):
        _fields_ = [("cx", ctypes.c_long), ("cy", ctypes.c_long)]

    hdc = ctypes.windll.user32.GetDC(0)
    hfont = ctypes.windll.gdi32.CreateFontA(font_size, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, font_name)
    hfont_old = ctypes.windll.gdi32.SelectObject(hdc, hfont)

    size = SIZE(0, 0)
    ctypes.windll.gdi32.GetTextExtentPoint32A(hdc, text, len(text), ctypes.byref(size))

    ctypes.windll.gdi32.SelectObject(hdc, hfont_old)
    ctypes.windll.gdi32.DeleteObject(hfont)

    return (size.cx, size.cy)



from bs4 import BeautifulSoup, Comment, SoupStrainer

#Remove all attributes from every tag (including children) in the given soup
#TODO: Remove some of the unnecessary code in the last line
def recursively_remove_attributes(soup):    
    for tag in soup.recursiveChildGenerator():
        if hasattr(tag, 'attrs'):
            tag.attrs = {key:value for key, value in tag.attrs.items()
                        if False}


def bs4_inner_html(bs4_element):
    if (bs4_element == None):
        return None
    else:
        return bs4_element.decode_contents(formatter="html")



#Some websites have memory leak issues where staying on the same browser tab for a while will gradually eat up the browser's memory until it crashes
#As a workaround, you can switch to a new tab every so often
def replace_current_tab_with_blank_tab(driver):
    driver.execute_script("window.open();")
    driver.switch_to.window(driver.window_handles[0])
    driver.close()
    driver.switch_to.window(driver.window_handles[0])


def get_element_value(parent_element, selection_method, selector, value_type):
    element_value = ''
    element = None

    try:
        if (selection_method == None):
            element = parent_element
            pass
        elif (selection_method == 'css'):
            element = parent_element.find_element_by_css_selector(selector)
            pass
        elif (selection_method == 'xpath'):
            element = parent_element.find_element_by_xpath(selector)
            pass
        else:
            element_value = ''
    
    except NoSuchElementException:
        element = None
    
    if (element != None):
        try:
            if (value_type == 'text'):
                element_value = element.text
            elif (value_type == 'value'):
                element_value = element.get_attribute('value')
            elif (value_type == 'href'):
                element_value = element.get_attribute('href')
            elif (value_type == 'inner_html'):
                element_value = element.get_attribute('innerHTML')
            elif (value_type == 'outer_html'):
                element_value = element.get_attribute('outerHTML')
            elif (value_type == 'select'):
                element_value = Select(element).first_selected_option.text
            elif (value_type == 'is_selected'):
                element_value = str(element.is_selected())
            else:
                element_value = ''

        except AttributeError:
            element_value = ''

    return element_value


def get_iframe_html(driver, css_selector):
    iframe_html = ''

    try:
        driver.switch_to.frame(driver.find_element_by_css_selector(css_selector))
        iframe_html = driver.find_element_by_tag_name("body").get_attribute('innerHTML')
        driver.switch_to.default_content()

    except NoSuchElementException:
        iframe_html = ''

    return iframe_html



import xml.etree.ElementTree as ET

#Recursively remove all namespaces from the lxml etree
def remove_all_lxml_namespaces(lxml_root):
    for element in lxml_root.getiterator():
        element.tag = etree.QName(element).localname
        etree.cleanup_namespaces(lxml_root)



def create_csv_cell(string):
    if (string == None):
        return '""'

    string = str(string)
    
    return ('"' + string.replace('"', '""') + '"')


#Converts an array of strings into a valid CSV row
def create_csv_row(cell_strings, include_newline):
    if (isinstance(cell_strings, list) == False):
        raise Exception("The first value passed to create_csv_row must be a list.")

    if (isinstance(include_newline, bool) == False):
        raise Exception("The second value passed to create_csv_row must be a bool.")
    
    row_string = ",".join([create_csv_cell(cell_string) for cell_string in cell_strings])

    if (include_newline == True):
        row_string += '\n'
    
    return row_string
